<?php
/**
 * 2007-2022 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    patworx multimedia GmbH <service@patworx.de>
 *  @copyright 2007-2022 patworx multimedia GmbH
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
class EasycreditClient
{
    public static $api_url = 'https://ratenkauf.easycredit.de';

    protected $easycreditCurl = '';

    protected $config = [
        'webshop_id' => '',
        'token' => '',
        'signature' => '',
    ];

    public static $endpoints = [
        'integrationcheck' => [
            'method' => 'POST',
            'url' => '/api/payment/v3/webshop/integrationcheck',
            'supportssignature' => true,
        ],
        'webshop' => [
            'method' => 'GET',
            'url' => '/api/payment/v3/webshop',
            'supportssignature' => true,
        ],
        'transaction' => [
            'method' => 'POST',
            'url' => '/api/payment/v3/transaction',
            'supportssignature' => true,
        ],
        'authtransaction' => [
            'method' => 'POST',
            'url' => '/api/payment/v3/transaction/%s/authorization',
            'supportssignature' => true,
        ],
        'fetchtransaction' => [
            'method' => 'GET',
            'url' => '/api/payment/v3/transaction/',
            'supportssignature' => true,
        ],
    ];

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->setConfig(
            'webshop_id',
            trim(Configuration::get('EASYCREDITRATENKAUF_API_KEY'))
        );
        $this->setConfig(
            'token',
            trim(Configuration::get('EASYCREDITRATENKAUF_API_TOKEN'))
        );
        $this->setConfig(
            'signature',
            trim(Configuration::get('EASYCREDITRATENKAUF_API_SIGNATURE_KEY'))
        );
        $this->easycreditCurl = new EasycreditCurl(
            $this->getConfig('webshop_id'),
            $this->getConfig('token')
        );
    }

    /**
     * @return EasycreditClient
     */
    public static function getInstance()
    {
        return new self();
    }

    /**
     * @param $var
     * @param $val
     */
    public function setConfig($var, $val)
    {
        $this->config[$var] = $val;
    }

    /**
     * @param $var
     *
     * @return false|string
     */
    public function getConfig($var)
    {
        if (isset($this->config[$var])) {
            return $this->config[$var];
        }

        return false;
    }

    /**
     * @return bool
     *
     * @throws Exception
     */
    public function integrationCheck()
    {
        $sent_message = json_encode([
            'message' => 'ratenkauf by easyCredit',
        ]);
        $callResponse = $this->call(
            self::$endpoints['integrationcheck'],
            $sent_message
        );

        return $callResponse['response'] == $sent_message;
    }

    /**
     * @return mixed
     *
     * @throws Exception
     */
    public function webshopInfo()
    {
        $callResponse = $this->call(
            self::$endpoints['webshop'],
            ''
        );

        return $callResponse['response'];
    }

    /**
     * @param $request
     *
     * @return mixed
     *
     * @throws Exception
     */
    public function initTransaction($request)
    {
        $callResponse = $this->call(
            self::$endpoints['transaction'],
            $request
        );

        return $callResponse['response'];
    }

    /**
     * @param $transaction_id
     *
     * @return mixed
     *
     * @throws Exception
     */
    public function fetchTransaction($transaction_id)
    {
        $callResponse = $this->call(
            self::$endpoints['fetchtransaction'],
            $transaction_id
        );

        return $callResponse['response'];
    }

    /**
     * @param $transaction_id
     *
     * @return statusCode
     *
     * @throws Exception
     */
    public function authorizeTransaction($transaction_id)
    {
        $callResponse = $this->call(
            self::$endpoints['authtransaction'],
            '',
            $transaction_id
        );

        return $callResponse['status'];
    }

    /**
     * @param array $endpoint
     * @param $payload
     * @param bool|string $sprintf
     *
     * @return array
     *
     * @throws Exception
     */
    protected function call(array $endpoint, $payload, $sprintf = false)
    {
        $url = self::$api_url . $endpoint['url'];
        if ($endpoint['method'] == 'GET') {
            $url .= $payload;
            $payload = '';
        }
        if ($sprintf) {
            $url = sprintf($url, $sprintf);
        }
        $postSignedHeaders = [
            'Content-Type:application/json',
        ];
        if ($this->getConfig('signature') != '' && $endpoint['supportssignature']) {
            $sha256Hash = hash(
                'sha256',
                $this->normalizePayload($payload) . $this->getConfig('signature')
            );
            $postSignedHeaders[] = 'Content-signature: sha256=' . $sha256Hash;
        }

        return $this->easycreditCurl->invokeCurl(
            $endpoint['method'],
            $url,
            $payload,
            $postSignedHeaders
        );
    }

    /**
     * @param $payload
     *
     * @return array|string|string[]
     */
    protected function normalizePayload($payload)
    {
        return str_replace([
            "\n", "\t", "\r",
        ], '', $payload);
    }
}
