<?php
/**
* 2007-2022 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author    patworx multimedia GmbH <service@patworx.de>
*  @copyright 2007-2022 patworx multimedia GmbH
*  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/
if (!defined('_PS_VERSION_')) {
    exit;
}

require_once dirname(__FILE__) . '/classes/EasycreditLog.php';
require_once dirname(__FILE__) . '/classes/EasycreditSession.php';
require_once dirname(__FILE__) . '/classes/EasycreditCurl.php';
require_once dirname(__FILE__) . '/classes/EasycreditClient.php';
require_once dirname(__FILE__) . '/classes/EasycreditWebshopInfo.php';
require_once dirname(__FILE__) . '/vendor/redirectionTrait.php';
require_once dirname(__FILE__) . '/classes/Structs/Base.php';
require_once dirname(__FILE__) . '/classes/Structs/Address.php';
require_once dirname(__FILE__) . '/classes/Structs/ArticleNumberItem.php';
require_once dirname(__FILE__) . '/classes/Structs/Bank.php';
require_once dirname(__FILE__) . '/classes/Structs/Contact.php';
require_once dirname(__FILE__) . '/classes/Structs/Customer.php';
require_once dirname(__FILE__) . '/classes/Structs/CustomerRelationship.php';
require_once dirname(__FILE__) . '/classes/Structs/Employment.php';
require_once dirname(__FILE__) . '/classes/Structs/InvoiceAddress.php';
require_once dirname(__FILE__) . '/classes/Structs/OrderDetails.php';
require_once dirname(__FILE__) . '/classes/Structs/RedirectLinks.php';
require_once dirname(__FILE__) . '/classes/Structs/ShippingAddress.php';
require_once dirname(__FILE__) . '/classes/Structs/ShoppingCartInformation.php';
require_once dirname(__FILE__) . '/classes/Structs/ShoppingCartInformationItem.php';
require_once dirname(__FILE__) . '/classes/Structs/Shopsystem.php';
require_once dirname(__FILE__) . '/classes/Structs/Transaction.php';

class Easycreditratenkauf extends PaymentModule
{
    public $easycreditClient;

    protected $config_form = false;

    private $_postSuccess = [];
    private $_postError = [];

    protected $min_amount = 200;
    protected $max_amount = 10000;

    public function __construct()
    {
        $this->name = 'easycreditratenkauf';
        $this->tab = 'payments_gateways';
        $this->version = '2.1.2';
        $this->author = 'TeamBank AG, developed by patworx multimedia GmbH';
        $this->need_instance = 1;
        $this->bootstrap = true;
        $this->module_key = '4100b32025aedaf0db43d3c491941f31';

        parent::__construct();

        $this->displayName = $this->l('easyCredit-Ratenkauf');
        $this->description = $this->l('Offer installment by easyCredit for your customers');

        $this->confirmUninstall = $this->l('Are you sure to uninstall this module?');

        $this->limited_countries = ['DE'];

        $this->limited_currencies = ['EUR'];

        $this->ps_versions_compliancy = ['min' => '1.7', 'max' => _PS_VERSION_];

        $this->easycreditClient = new EasycreditClient();
    }

    /**
     * @return bool
     *
     * @throws Exception
     */
    public function install()
    {
        if (extension_loaded('curl') == false) {
            $this->_errors[] = $this->l('You have to enable the cURL extension on your server to install this module');

            return false;
        }

        $iso_code = Country::getIsoById(Configuration::get('PS_COUNTRY_DEFAULT'));

        if (in_array($iso_code, $this->limited_countries) == false) {
            $this->_errors[] = $this->l('This module is not available in your country');

            return false;
        }

        include dirname(__FILE__) . '/sql/install.php';

        Tools::clearSmartyCache();
        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();

        $sf2Refresh = new \PrestaShopBundle\Service\Cache\Refresh();
        $sf2Refresh->addCacheClear(_PS_MODE_DEV_ ? 'dev' : 'prod');
        $sf2Refresh->execute();

        return parent::install() &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('displayBackOfficeHeader') &&
            $this->registerHook('paymentReturn') &&
            $this->registerHook('displayShoppingCart') &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayAdminOrder') &&
            $this->registerHook('displayProductButtons');
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        include dirname(__FILE__) . '/sql/uninstall.php';

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        if (((bool) Tools::isSubmit('submitEasycreditratenkaufModule')) == true) {
            $this->postProcess();
            if (count($this->_postSuccess)) {
                $this->context->smarty->assign(['postSuccess' => $this->_postSuccess]);
            }
            if (count($this->_postError)) {
                $this->context->smarty->assign(['postError' => $this->_postError]);
            }
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('ec_form', $this->renderForm());
        $this->context->smarty->assign('is_valid_credentials', $this->easycreditClient->integrationCheck());

        return $this->context->smarty->fetch($this->local_path . 'views/templates/admin/config_plugin.tpl');
    }

    /**
     * @return string
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitEasycreditratenkaufModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }

    /**
     * @return array[]
     */
    protected function getConfigForm()
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => [
                    [
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Please enter your easyCredit-Ratenkauf Webshop-ID'),
                        'name' => 'EASYCREDITRATENKAUF_API_KEY',
                        'label' => $this->l('Webshop-ID'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Please enter your easyCredit-Ratenkauf API-Password'),
                        'name' => 'EASYCREDITRATENKAUF_API_TOKEN',
                        'label' => $this->l('API-Password'),
                    ],
                    [
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Please enter only if you activated this feature in your merchant portal'),
                        'name' => 'EASYCREDITRATENKAUF_API_SIGNATURE_KEY',
                        'label' => $this->l('API-Signature Key'),
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * @return array
     */
    protected function getConfigFormValues()
    {
        return [
            'EASYCREDITRATENKAUF_API_KEY' => Configuration::get('EASYCREDITRATENKAUF_API_KEY'),
            'EASYCREDITRATENKAUF_API_TOKEN' => Configuration::get('EASYCREDITRATENKAUF_API_TOKEN'),
            'EASYCREDITRATENKAUF_API_SIGNATURE_KEY' => Configuration::get('EASYCREDITRATENKAUF_API_SIGNATURE_KEY'),
        ];
    }

    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, trim(Tools::getValue($key)));
        }
        $this->_postSuccess[] = $this->l('Settings updated');
    }

    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('configure') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
        $this->context->controller->addJS($this->_path . 'views/js/minimize.js');
    }

    /**
     * @return false|string
     */
    public function hookDisplayHeader()
    {
        $htmlreturn = false;
        if (isset($this->context->cookie->easycredit_error_message)) {
            $this->context->controller->errors[] = $this->context->cookie->easycredit_error_message;
            unset($this->context->cookie->easycredit_error_message);
        }
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');

        Media::addJsDef(
            [
                'easycredit' => [
                    'submit_button' => $this->l('Continue to easycredit'),
                ],
            ]
        );

        return $this->display(__FILE__, 'views/templates/hook/header.tpl');
    }

    /**
     * @param $params
     *
     * @return array|\PrestaShop\PrestaShop\Core\Payment\PaymentOption[]
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return [];
        }
        $total = $params['cart']->getOrderTotal(true);

        if (!$this->ratenkaufIsAvailable($total)) {
            return [];
        }

        $this->smarty->assign('zustimmungtext', $this->getZustimmungText());

        $newOption = new PrestaShop\PrestaShop\Core\Payment\PaymentOption();
        $newOption->setCallToActionText($this->l('easyCredit-Ratenkauf'))
            ->setModuleName('easycreditratenkauf')
            ->setAction($this->context->link->getModuleLink($this->name, 'redirect', [], true))
            ->setLogo(Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/rk_logo.jpg'))
            ->setAdditionalInformation($this->display(__FILE__, 'views/templates/hook/hookPaymentOptionAdditionalInfo.tpl'));
        $payment_options = [$newOption];

        return $payment_options;
    }

    /**
     * @return false|string
     */
    protected function getZustimmungText()
    {
        return EasycreditWebshopInfo::getInstance()->get('privacyApprovalForm');
    }

    /**
     * @param $params
     *
     * @return false|string
     */
    public function hookDisplayProductButtons($params)
    {
        if (is_object($params['product']) && $params['product'] instanceof Product) {
            $this->context->smarty->assign('interest_calculator_price', (float) $params['product']->getPrice());
        } else {
            $this->context->smarty->assign('interest_calculator_price', (float) $params['product']['price_amount']);
        }

        // if (!$this->ratenkaufIsAvailable($this->context->smarty->getTemplateVars('interest_calculator_price'))) {
        //    return false;
        // }

        $this->context->smarty->assign('webshopID', Configuration::get('EASYCREDITRATENKAUF_API_KEY'));

        return $this->display(__FILE__, 'views/templates/hook/display_product_button.tpl');
    }

    /**
     * @param $params
     *
     * @return false|string
     */
    public function hookDisplayShoppingCart($params)
    {
        $this->context->smarty->assign('interest_calculator_price', (float) $params['cart']->getOrderTotal(true));
        $this->context->smarty->assign('webshopID', Configuration::get('EASYCREDITRATENKAUF_API_KEY'));

        return $this->display(__FILE__, 'views/templates/hook/display_shopping_cart.tpl');
    }

    /**
     * @param $amount
     *
     * @return bool
     */
    public function ratenkaufIsAvailable($amount)
    {
        if ($amount > EasycreditWebshopInfo::getInstance()->get('maxFinancingAmount') ||
            $amount < EasycreditWebshopInfo::getInstance()->get('minFinancingAmount')) {
            return false;
        }

        return true;
    }

    /**
     * Admin actions and log-view
     *
     * @param $params
     */
    public function hookDisplayAdminOrder($params)
    {
        $order = new Order((int) $params['id_order']);
        if ($order->module != $this->name) {
            return;
        }
        if (version_compare(_PS_VERSION_, '1.7.7', '>=')) {
            $this->context->smarty->assign('isHigher176', true);
        } else {
            $this->context->smarty->assign('isHigher176', false);
        }
        $this->context->smarty->assign('transactions', EasycreditLog::getDataForOrderId((int) $order->id));

        return $this->display(__FILE__, 'views/templates/admin/hook_order_history.tpl');
    }

    /**
     * @param $ps_cart_id
     * @param $ps_customer_id
     * @param $url
     * @param $easycredit_vorgang
     * @param $request
     * @param $response
     *
     * @throws PrestaShopException
     */
    public static function createLog($ps_cart_id, $ps_customer_id, $url, $easycredit_vorgang, $request, $response)
    {
        $easycreditLog = new EasycreditLog();
        $easycreditLog->ps_cart_id = (int) $ps_cart_id;
        $easycreditLog->ps_customer_id = (int) $ps_customer_id;
        $easycreditLog->url = $url;
        $easycreditLog->easycredit_vorgang = pSQL($easycredit_vorgang);
        $easycreditLog->request = pSQL($request);
        $easycreditLog->response = pSQL($response);
        $easycreditLog->save();
    }
}
